@@ -74,7 +74,7 @@
 #' * function: accept two equal-length numeric vectors of adjusted predictions (`hi` and `lo`) and returns a vector of contrasts of the same length, or a unique numeric value.
 #'   - See the Transformations section below for examples of valid functions.
 #' @param transform string or function. Transformation applied to unit-level estimates and confidence intervals just before the function returns results. Functions must accept a vector and return a vector of the same length. Support string shortcuts: "exp", "ln"
-#' @param equivalence Numeric vector of length 2: bounds used for the two-one-sided test (TOST) of equivalence, and for the non-inferiority and non-superiority tests. See Details section below.
+#' @param equivalence Numeric vector of length 2: bounds used for the two-one-sided test (TOST) of equivalence, and for the non-inferiority and non-superiority tests. For bayesian models, this report the proportion of posterior draws in the interval and the ROPE. See Details section below.
 #' @param by Aggregate unit-level estimates (aka, marginalize, average over). Valid inputs:
 #'   - `FALSE`: return the original unit-level estimates.
 #'   - `TRUE`: aggregate estimates for each term.
@@ -98,8 +98,6 @@
 #' @template return
 #'
 #' @examplesIf interactive() || isTRUE(Sys.getenv("R_DOC_BUILD") == "true")
-#' library(marginaleffects)
-#'
 #' # Linear model
 #' tmp <- mtcars
 #' tmp$am <- as.logical(tmp$am)
@@ -235,8 +233,54 @@ comparisons <- function(
     numderiv = "fdforward",
     ...
 ) {
-    call_attr <- construct_call(model, "comparisons")
+    # init
+    if (inherits(model, "marginaleffects_internal")) {
+        mfx <- model
+    } else {
+        # pass through ... to avoid calling `get_modeldata()` in inferences()
+        if ("modeldata" %in% ...names()) {
+            modeldata <- ...get("modeldata")
+        } else {
+            modeldata <- NULL
+        }
+        call <- construct_call(model, "comparisons")
+        model <- sanitize_model(model, 
+            call = call, 
+            newdata = newdata, 
+            wts = wts, 
+            vcov = vcov, 
+            by = by, ...)
+        mfx <- new_marginaleffects_internal(
+            call = call,
+            model = model,
+            modeldata = modeldata,
+            by = by,
+            comparison = comparison,
+            cross = cross,
+            eps = eps
+        )
+    }
 
+    scall <- rlang::enquo(newdata)
+    mfx <- add_newdata(mfx, 
+        scall, 
+        newdata = newdata, 
+        by = by, 
+        wts = wts, 
+        cross = cross, 
+        comparison = comparison)
+
+    # sanity checks
+    dots <- list(...)
+    sanity_dots(mfx@model, ...)
+
+    # multiple imputation
+    if (inherits(mfx@model, c("mira", "amest"))) {
+        out <- process_imputation(mfx)
+        return(out)
+    }
+
+    # inferences() dispatch
     methods <- c("rsample", "boot", "fwb", "simulation")
     if (isTRUE(checkmate::check_choice(vcov, methods))) {
         inferences_method <- vcov
@@ -245,327 +289,169 @@ comparisons <- function(
         inferences_method <- NULL
     }
 
-    # multiple imputation
-    if (inherits(model, c("mira", "amest"))) {
-        out <- process_imputation(model, call_attr)
-        return(out)
-    }
-
-    dots <- list(...)
-
-    # extracting modeldata repeatedly is slow.
-    if ("modeldata" %in% ...names()) {
-        modeldata <- call_attr[["modeldata"]] <- ...get("modeldata")
-    } else {
-        modeldata <- get_modeldata(
-            model,
-            additional_variables = by,
-            modeldata = dots[["modeldata"]],
-            wts = wts
-        )
-        if (isTRUE(checkmate::check_data_frame(modeldata))) {
-            call_attr[["modeldata"]] <- modeldata
-        }
-    }
-
     # very early, before any use of newdata
-    # if `newdata` is a call to `typical` or `counterfactual`, insert `model`
-    scall <- rlang::enquo(newdata)
-    newdata <- sanitize_newdata_call(scall, newdata, model, by = by)
-    if (is.null(modeldata) && isTRUE(checkmate::check_data_frame(newdata))) {
-        modeldata <- newdata
-    }
+    newdata <- mfx@newdata
 
-    # required by stubcols later, but might be overwritten
-    bycols <- NULL
-
-    # sanity checks
-    sanity_dots(model, ...)
-    conf_level <- sanitize_conf_level(conf_level, ...)
+    # misc
+    mfx@conf_level <- sanitize_conf_level(conf_level, ...)
+    mfx <- add_numderiv(mfx, numderiv)
     checkmate::assert_number(eps, lower = 1e-10, null.ok = TRUE)
-    numderiv <- sanitize_numderiv(numderiv)
-    model <- sanitize_model(
-        model = model,
-        newdata = newdata,
-        wts = wts,
-        vcov = vcov,
-        by = by,
-        calling_function = "comparisons",
-        ...
-    )
-    df <- sanitize_df(
-        df = df,
-        model = model,
-        newdata = newdata,
-        vcov = vcov,
-        by = by,
-        hypothesis = hypothesis
-    )
-    cross <- sanitize_cross(cross, variables, model)
-    type <- sanitize_type(
-        model = model,
+
+    # misc sanitation
+    mfx <- add_by(mfx, by)
+    sanity_reserved(mfx)
+
+    cross <- sanitize_cross(cross, variables, mfx@model)
+    mfx@type <- sanitize_type(
+        model = mfx@model,
         type = type,
-        calling_function = "comparisons"
+        calling_function = mfx@calling_function
     )
     sanity_comparison(comparison)
 
     # transforms
-    comparison_label <- transform_label <- NULL
-    if (is.function(comparison)) {
-        comparison_label <- deparse(substitute(comparison))
-    }
-    if (is.function(transform)) {
-        transform_label <- deparse(substitute(transform))
-        transform <- sanitize_transform(transform)
-        names(transform) <- transform_label
-    } else {
-        transform <- sanitize_transform(transform)
-        transform_label <- names(transform)
-    }
-
-    newdata <- sanitize_newdata(
-        model = model,
-        newdata = newdata,
-        modeldata = modeldata,
-        by = by,
-        wts = wts
-    )
+    transform <- sanitize_transform(transform)
 
     # after sanitize_newdata
-    if (is.null(modeldata) && isTRUE(checkmate::check_data_frame(newdata))) {
-        modeldata <- call_attr[["modeldata"]] <- newdata
-    }
-
-    # after sanitize_newdata
-    sanity_by(by, newdata)
-
-    # after sanity_by
-    newdata <- dedup_newdata(
-        model = model,
-        newdata = newdata,
-        wts = wts,
-        by = by,
-        cross = cross,
-        comparison = comparison
-    )
-    if (isFALSE(wts) && "marginaleffects_wts_internal" %in% colnames(newdata)) {
-        wts <- "marginaleffects_wts_internal"
-    }
-
-    # after sanitize_newdata
-    # after dedup_newdata
-    variables_list <- sanitize_variables(
-        model = model,
-        newdata = newdata,
-        modeldata = modeldata,
+    mfx <- add_variables(
         variables = variables,
-        cross = cross,
-        by = by,
-        comparison = comparison,
-        eps = eps
+        mfx = mfx
     )
 
     # get dof before transforming the vcov arg
-    # get_degrees_of_freedom() produces a weird warning on non lmerMod. We can skip them
+    # add_degrees_of_freedom() produces a weird warning on non lmerMod. We can skip them
     # because get_vcov() will produce an informative error later.
-    df <- get_degrees_of_freedom(model = model, df = df, newdata = newdata)
+    mfx <- add_degrees_of_freedom(mfx = mfx, df = df, by = by, hypothesis = hypothesis, vcov = vcov)
 
-    vcov_false <- isFALSE(vcov)
-    vcov.type <- get_vcov_label(vcov)
-    vcov <- get_vcov(model, vcov = vcov, type = type, ...)
+    mfx@vcov_type <- get_vcov_label(vcov)
+    mfx@vcov_model <- get_vcov(mfx@model, vcov = vcov, type = mfx@type, ...)
 
-    predictors <- variables_list$conditional
+    predictors <- mfx@variables
+
+    mfx <- add_hypothesis(mfx, hypothesis)
 
     ############### sanity checks are over
 
-    # after inferences dispatch
-    tmp <- sanitize_hypothesis(hypothesis, ...)
-    hypothesis <- tmp$hypothesis
-    hypothesis_null <- tmp$hypothesis_null
-    hypothesis_direction <- tmp$hypothesis_direction
-
     # compute contrasts and standard errors
     args <- list(
-        model = model,
-        newdata = newdata,
+        mfx = mfx,
         variables = predictors,
-        cross = cross,
-        modeldata = modeldata
+        cross = cross
     )
     dots[["modeldata"]] <- NULL # dont' pass twice
     args <- utils::modifyList(args, dots)
-    contrast_data <- do.call("get_contrast_data", args)
+    contrast_data <- do.call("get_comparisons_data", args)
 
     args <- list(
-        model,
-        newdata = newdata,
+        mfx = mfx,
         variables = predictors,
-        type = type,
+        type = mfx@type,
         original = contrast_data[["original"]],
         hi = contrast_data[["hi"]],
         lo = contrast_data[["lo"]],
-        wts = contrast_data[["original"]][["marginaleffects_wts_internal"]],
         by = by,
         cross = cross,
-        hypothesis = hypothesis,
-        modeldata = modeldata
+        hypothesis = mfx@hypothesis
     )
     args <- utils::modifyList(args, dots)
-    mfx <- do.call("get_contrasts", args)
-
-    hyp_by <- attr(mfx, "hypothesis_function_by")
+    cmp <- do.call("get_comparisons", args)
 
     # bayesian posterior
-    if (!is.null(attr(mfx, "posterior_draws"))) {
-        draws <- attr(mfx, "posterior_draws")
-        J <- NULL
+    mfx@draws <- attr(cmp, "posterior_draws")
 
-        # standard errors via delta method
-    } else if (!vcov_false && isTRUE(checkmate::check_matrix(vcov))) {
-        idx <- intersect(colnames(mfx), c("group", "term", "contrast"))
-        idx <- mfx[, (idx), drop = FALSE]
+    # standard errors via delta method
+    if (is.null(mfx@draws) && 
+        !isFALSE(vcov) && 
+        isTRUE(checkmate::check_matrix(mfx@vcov_model))) {
+
+        idx <- intersect(colnames(cmp), c("group", "term", "contrast"))
+        idx <- cmp[, (idx), drop = FALSE]
+        fun <- function(...) {
+            get_comparisons(..., verbose = FALSE)$estimate
+        }
         args <- list(
-            model,
-            vcov = vcov,
-            type = type,
-            FUN = get_se_delta_contrasts,
-            newdata = newdata,
+            mfx = mfx,
+            model_perturbed = mfx@model,
+            vcov = mfx@vcov_model,
+            type = mfx@type,
+            FUN = fun,
             index = idx,
             variables = predictors,
-            hypothesis = hypothesis,
+            hypothesis = mfx@hypothesis,
             hi = contrast_data$hi,
             lo = contrast_data$lo,
             original = contrast_data$original,
-            by = by,
-            eps = eps,
-            cross = cross,
-            numderiv = numderiv,
-            comparison = comparison,
-            calling_function = "comparisons"
+            numderiv = numderiv
         )
         args <- utils::modifyList(args, dots)
         se <- do.call("get_se_delta", args)
-        J <- attr(se, "jacobian")
-        attr(se, "jacobian") <- NULL
-        mfx$std.error <- as.numeric(se)
-        draws <- NULL
-
-        # no standard error
-    } else {
-        J <- draws <- NULL
+        mfx@jacobian <- attr(se, "jacobian")
+        cmp$std.error <- as.vector(as.numeric(se)) # drop attributes
+        mfx@draws <- NULL
     }
 
     # merge original data back in
-    if ((is.null(by) || isFALSE(by)) && "rowid" %in% colnames(mfx)) {
-        if ("rowid" %in% colnames(newdata)) {
+    if (isFALSE(by) && "rowid" %in% colnames(cmp)) {
+        if ("rowid" %in% colnames(mfx@newdata)) {
             idx <- c(
                 "rowid",
                 "rowidcf",
                 "term",
                 "contrast",
                 "by",
-                setdiff(colnames(contrast_data$original), colnames(mfx))
+                setdiff(colnames(contrast_data$original), colnames(cmp))
             )
             idx <- intersect(idx, colnames(contrast_data$original))
             tmp <- contrast_data$original[, ..idx, drop = FALSE]
             # contrast_data is duplicated to compute contrasts for different terms or pairs
-            bycols <- intersect(colnames(tmp), colnames(mfx))
+            bycols <- intersect(colnames(tmp), colnames(cmp))
             idx <- duplicated(tmp, by = bycols)
             tmp <- tmp[!idx]
-            mfx <- merge(mfx, tmp, all.x = TRUE, by = bycols, sort = FALSE)
+            cmp <- merge(cmp, tmp, all.x = TRUE, by = bycols, sort = FALSE)
             # HACK: relies on NO sorting at ANY point
         } else {
-            idx <- setdiff(colnames(contrast_data$original), colnames(mfx))
-            mfx <- data.table(mfx, contrast_data$original[, ..idx])
+            idx <- setdiff(colnames(contrast_data$original), colnames(cmp))
+            cmp <- data.table(cmp, contrast_data$original[, ..idx])
         }
     }
 
     # meta info
-    mfx <- get_ci(
-        mfx,
-        conf_level = conf_level,
-        df = df,
-        draws = draws,
-        estimate = "estimate",
-        hypothesis_null = hypothesis_null,
-        hypothesis_direction = hypothesis_direction,
-        model = model
-    )
+    cmp <- get_ci(cmp, mfx)
 
     # clean rows and columns
     # WARNING: we cannot sort rows at the end because `get_hypothesis()` is
     # applied in the middle, and it must already be sorted in the final order,
     # otherwise, users cannot know for sure what is going to be the first and
     # second rows, etc.
-    mfx <- sort_columns(mfx, newdata, by)
-
-    # bayesian draws
-    attr(mfx, "posterior_draws") <- draws
+    cmp <- sort_columns(cmp, mfx@newdata, by)
 
     # equivalence tests
-    mfx <- equivalence(mfx, equivalence = equivalence, df = df, ...)
+    cmp <- equivalence(cmp, equivalence = equivalence, df = mfx@df, draws = mfx@draws, ...)
 
     # after draws attribute
-    mfx <- backtransform(mfx, transform)
+    cmp <- backtransform(cmp, transform, draws = mfx@draws)
+    new_draws <- attr(cmp, "posterior_draws") # important!
+    if (!is.null(new_draws)) mfx@draws <- new_draws
 
-    # save as attribute and not column
-    if (!all(is.na(mfx[["marginaleffects_wts_internal"]]))) {
-        marginaleffects_wts_internal <- mfx[["marginaleffects_wts_internal"]]
-    } else {
-        marginaleffects_wts_internal <- NULL
-    }
-    mfx[["marginaleffects_wts_internal"]] <- NULL
+    # remove weights column (now handled by add_attributes)
+    cmp[["marginaleffects_wts_internal"]] <- NULL
 
-    out <- mfx
+    out <- cmp
 
     data.table::setDF(out)
 
-    out <- set_marginaleffects_attributes(
-        out,
-        get_marginaleffects_attributes(
-            newdata,
-            include_regex = "^newdata.*class|explicit|matrix|levels"
-        )
-    )
+    class(out) <- c("comparisons", class(out))
 
-    # Global option for lean return object
-    lean <- getOption("marginaleffects_lean", default = FALSE)
-
-    # Only add (potentially large) attributes if lean is FALSE
-    # extra attributes needed for print method, even with lean return object
-    attr(out, "conf_level") <- conf_level
-    attr(out, "by") <- by
-    attr(out, "lean") <- lean
-    attr(out, "vcov.type") <- vcov.type
-    if (isTRUE(lean)) {
-        for (a in setdiff(names(attributes(out)), c("names", "row.names", "class"))) {
-            attr(out, a) <- NULL
-        }
-    } else {
-        # other attributes
-        attr(out, "newdata") <- newdata
-        attr(out, "call") <- call_attr
-        attr(out, "type") <- type
-        attr(out, "model_type") <- class(model)[1]
-        attr(out, "model") <- model
-        attr(out, "variables") <- predictors
-        attr(out, "jacobian") <- J
-        attr(out, "vcov") <- vcov
-        attr(out, "vcov.type") <- vcov.type
-        attr(out, "weights") <- marginaleffects_wts_internal
-        attr(out, "comparison") <- comparison
-        attr(out, "transform") <- transform[[1]]
-        attr(out, "comparison_label") <- comparison_label
-        attr(out, "hypothesis_by") <- hyp_by
-        attr(out, "transform_label") <- transform_label
-
-        if (inherits(model, "brmsfit")) {
-            insight::check_if_installed("brms")
-            attr(out, "nchains") <- brms::nchains(model)
-        }
+    # before add_attributes()
+    if (inherits(mfx@model, "brmsfit")) {
+        insight::check_if_installed("brms")
+        mfx@draws_chains <- brms::nchains(mfx@model) 
     }
 
-    class(out) <- c("comparisons", class(out))
+    # class before prune
+    out <- add_attributes(out, mfx)
+    out <- prune_attributes(out)
+
 
     if (!is.null(inferences_method)) {
         out <- inferences(out, method = inferences_method)
