@@ -46,9 +46,10 @@ get_predict.multinom <- function(
     model,
     newdata = insight::get_data(model),
     type = "probs",
-    ...
-) {
-    type <- sanitize_type(model, type, calling_function = "predictions")
+    mfx = NULL,
+    ...) {
+    calling_function <- if (!is.null(mfx)) mfx@calling_function else "predictions"
+    type <- sanitize_type(model, type, calling_function = calling_function)
 
     is_latent <- is_mclogit <- is_nnet <- FALSE
     if (isTRUE(type == "latent") && inherits(model, c("mblogit", "mclogit"))) {
@@ -85,7 +86,7 @@ get_predict.multinom <- function(
             colnames(pred)[1] <- missing_level
             pred <- pred - rowMeans(pred)
         } else {
-            insight::format_error(
+            stop_sprintf(
                 "Unable to compute predictions on the latent scale."
             )
         }
@@ -98,18 +99,12 @@ get_predict.multinom <- function(
     }
 
     # matrix with outcome levels as columns
-    out <- data.frame(
+    out <- data.table(
         group = rep(colnames(pred), each = nrow(pred)),
         estimate = c(pred)
     )
     out$group <- group_to_factor(out$group, model)
-
-    # usually when `newdata` is supplied by `comparisons`
-    if ("rowid" %in% colnames(newdata)) {
-        out$rowid <- rep(newdata$rowid, times = ncol(pred))
-    } else {
-        out$rowid <- rep(seq_len(nrow(pred)), times = ncol(pred))
-    }
+    out <- add_rowid(out, newdata)
 
     return(out)
 }
