@@ -4,33 +4,27 @@ get_predict.rq <- function(
     model,
     newdata = insight::get_data(model),
     type = NULL,
-    ...
-) {
+    ...) {
     # type argument of the method is used to specify confidence interval type
     insight::check_if_installed("quantreg")
-    if (
-        isTRUE(
-            getOption("marginaleffects_linalg", default = "RcppEigen") == "RcppEigen"
-        )
-    ) {
+    if (isTRUE(getOption("marginaleffects_linalg", default = "RcppEigen") == "RcppEigen")) {
         MM <- attr(newdata, "marginaleffects_model_matrix")
         if (isTRUE(checkmate::check_matrix(MM))) {
             beta <- get_coef(model)
             out <- hush(eigenMatMult(MM, beta))
             if (isTRUE(checkmate::check_numeric(out, len = nrow(newdata)))) {
-                out <- data.frame(rowid = newdata$rowid, estimate = out)
-                return(out)
+                out <- data.table(estimate = out)
+                out <- add_rowid(out, newdata)
             } else {
-                out <- data.frame(rowid = seq_len(length(out)), estimate = out)
+                out <- data.table(estimate = out)
+                out <- add_rowid(out, newdata)
             }
+            return(out)
         }
     }
     out <- quantreg::predict.rq(model, newdata = newdata, ...)
-    if (isTRUE(checkmate::check_numeric(out, len = nrow(newdata)))) {
-        out <- data.frame(rowid = newdata$rowid, estimate = out)
-    } else {
-        out <- data.frame(rowid = seq_len(length(out)), estimate = out)
-    }
+    out <- data.table(estimate = out)
+    out <- add_rowid(out, newdata)
     return(out)
 }
 
