@@ -3,7 +3,7 @@
 get_vcov.orm <- function(model, vcov = NULL, ...) {
     if (!is.null(vcov) && !isTRUE(checkmate::check_flag(vcov))) {
         msg <- "The `vcov` argument is not supported for models of this class."
-        insight::format_error(msg)
+        stop_sprintf(msg)
     }
     vcov <- sanitize_vcov(model, vcov)
     out <- stats::vcov(model, intercepts = "all")
@@ -17,10 +17,11 @@ get_predict.rms <- function(
     model,
     newdata = insight::get_data(model),
     type = NULL,
-    ...
-) {
+    mfx = NULL,
+    ...) {
     if (is.null(type)) {
-        type <- sanitize_type(model, type, calling_function = "predictions")
+        calling_function <- if (!is.null(mfx)) mfx@calling_function else "predictions"
+        type <- sanitize_type(model, type, calling_function = calling_function)
     }
     if (inherits(newdata, "tbl_df")) {
         warning("Converting `newdata` from tibble to data.frame.", call. = FALSE)
@@ -28,7 +29,9 @@ get_predict.rms <- function(
     }
 
     # {rms} predict methods break on additional arguments
-    get_predict.default(model, newdata = newdata, type = type)
+    out <- get_predict.default(model, newdata = newdata, type = type)
+    out <- add_rowid(out, newdata)
+    return(out)
 }
 
 #' @rdname get_predict
