@@ -11,6 +11,7 @@ utils::globalVariables(c(
     "..cl",
     "..col_newdata",
     "..col_x",
+    "..keepers",
     "..cols",
     "..hypothesis_by",
     "..idx",
@@ -58,6 +59,7 @@ utils::globalVariables(c(
     "marginaleffects_contrast_lo",
     "marginaleffects_function",
     "marginaleffects_internal_label",
+    "marginaleffects_internal_rowid",
     "marginaleffects_lab",
     "marginaleffects_ter",
     "marginaleffects_unique_labels",
@@ -68,6 +70,8 @@ utils::globalVariables(c(
     "part",
     "patterns",
     "position",
+    "pred_hi",
+    "pred_lo",
     "predicted",
     "predicted_hi",
     "predicted_lo",
