@@ -49,8 +49,8 @@
 #'
 #' # marginal predictions on a counterfactual grid
 #' plot_predictions(mod,
-#'   by = "am",
-#'   newdata = datagrid(am = 0:1, grid_type = "counterfactual")
+#'     by = "am",
+#'     newdata = datagrid(am = 0:1, grid_type = "counterfactual")
 #' )
 #'
 plot_predictions <- function(
@@ -67,20 +67,33 @@ plot_predictions <- function(
     rug = FALSE,
     gray = getOption("marginaleffects_plot_gray", default = FALSE),
     draw = TRUE,
-    ...
-) {
+    ...) {
     checkmate::assert_number(points, lower = 0, upper = 1)
 
-    if (inherits(model, "mira") && is.null(newdata)) {
+    if (inherits(model, c("mira", "amest"))) {
+        msg <- "This function does not support multiple imputation. Call `predictions()` or `avg_predictions()` instead. These functions return easy to plot data frames."
+        stop_sprintf(msg)
+    }
+
+
+    # init
+    call <- construct_call(model, "comparisons")
+    model <- sanitize_model(model, call = call, newdata = newdata, wts = wts, vcov = vcov, by = by, ...)
+    mfx <- new_marginaleffects_internal(
+        call = call,
+        model = model
+    )
+
+    if (inherits(mfx@model, "mira") && is.null(newdata)) {
         msg <- "Please supply a data frame to the `newdata` argument explicitly."
-        insight::format_error(msg)
+        stop_sprintf(msg)
     }
 
     # order of the first few paragraphs is important
     scall <- rlang::enquo(newdata)
-    newdata <- sanitize_newdata_call(scall, newdata, model, by = by)
+    newdata <- sanitize_newdata_call(scall, newdata, mfx = mfx, by = by)
     if (!isFALSE(wts) && is.null(by)) {
-        insight::format_error("The `wts` argument requires a `by` argument.")
+        stop_sprintf("The `wts` argument requires a `by` argument.")
     }
     checkmate::assert_character(by, null.ok = TRUE)
 
@@ -94,40 +107,33 @@ plot_predictions <- function(
     )
     if ((!is.null(condition) && !is.null(by)) || (is.null(condition) && is.null(by))) {
         msg <- "One of the `condition` and `by` arguments must be supplied, but not both."
-        insight::format_error(msg)
+        stop_sprintf(msg)
     }
 
-    modeldata <- get_modeldata(
-        model,
-        additional_variables = c(names(condition), by),
-        wts = wts
-    )
-
     # mlr3 and tidymodels
-    if (is.null(modeldata) || nrow(modeldata) == 0) {
-        modeldata <- newdata
+    if (is.null(mfx@modeldata) || nrow(mfx@modeldata) == 0) {
+        mfx@modeldata <- newdata
     }
 
     # conditional
     if (!is.null(condition)) {
         condition <- sanitize_condition(
-            model,
-            condition,
-            variables = NULL,
-            modeldata = modeldata
+            mfx = mfx,
+            condition = condition,
+            variables = NULL
         )
         v_x <- condition$condition1
         v_color <- condition$condition2
         v_facet_1 <- condition$condition3
         v_facet_2 <- condition$condition4
         datplot <- predictions(
-            model,
+            mfx@model,
             newdata = condition$newdata,
             type = type,
             vcov = vcov,
             conf_level = conf_level,
             transform = transform,
-            modeldata = modeldata,
+            modeldata = mfx@modeldata,
             wts = wts,
             ...
         )
@@ -139,20 +145,19 @@ plot_predictions <- function(
         condition <- NULL
 
         newdata <- sanitize_newdata(
-            model = model,
+            mfx = mfx,
             newdata = newdata,
-            modeldata = modeldata,
             by = by,
             wts = wts
         )
 
         # tidymodels & mlr3
-        if (is.null(modeldata)) {
-            modeldata <- newdata
+        if (is.null(mfx@modeldata)) {
+            mfx@modeldata <- newdata
         }
 
         datplot <- predictions(
-            model,
+            mfx@model,
             by = by,
             type = type,
             vcov = vcov,
@@ -160,7 +165,7 @@ plot_predictions <- function(
             wts = wts,
             transform = transform,
             newdata = newdata,
-            modeldata = modeldata,
+            modeldata = mfx@modeldata,
             ...
         )
         v_x <- by[[1]]
@@ -169,10 +174,7 @@ plot_predictions <- function(
         v_facet_2 <- hush(by[[4]])
     }
 
-    dv <- unlist(
-        insight::find_response(model, combine = TRUE),
-        use.names = FALSE
-    )[1]
+    dv <- mfx@variable_names_response
     datplot <- plot_preprocess(
         datplot,
         v_x = v_x,
@@ -180,13 +182,12 @@ plot_predictions <- function(
         v_facet_1 = v_facet_1,
         v_facet_2 = v_facet_2,
         condition = condition,
-        modeldata = modeldata
+        mfx = mfx
     )
 
     # return immediately if the user doesn't want a plot
     if (isFALSE(draw)) {
         out <- as.data.frame(datplot)
-        attr(out, "posterior_draws") <- attr(datplot, "posterior_draws")
         return(out)
     }
 
@@ -199,10 +200,10 @@ plot_predictions <- function(
         v_facet_1 = v_facet_1,
         v_facet_2 = v_facet_2,
         points = points,
-        modeldata = modeldata,
         dv = dv,
         rug = rug,
-        gray = gray
+        gray = gray,
+        mfx = mfx
     )
 
     p <- p +
@@ -214,8 +215,5 @@ plot_predictions <- function(
             linetype = v_color
         )
 
-    # attach model data for each of use
-    attr(p, "modeldata") <- modeldata
-
     return(p)
 }
