@@ -14,24 +14,30 @@ sanitize_model_specific.default <- function(
     model,
     vcov = NULL,
     calling_function = "marginaleffects",
-    ...
-) {
+    ...) {
     return(model)
 }
 
 
-sanity_model_supported_class <- function(model) {
+sanity_model_supported_class <- function(model, custom = TRUE) {
     checkmate::assert_character(
         getOption("marginaleffects_model_classes", default = NULL),
         null.ok = TRUE
     )
-    custom_classes <- getOption("marginaleffects_model_classes", default = NULL)
-    custom_classes <- as.list(custom_classes)
+
+    if (isTRUE(custom)) {
+        custom_classes <- getOption("marginaleffects_model_classes", default = NULL)
+        custom_classes <- as.list(custom_classes)
+    } else {
+        custom_classes <- NULL
+    }
+
     supported <- append(
         custom_classes,
         list(
+            "marginaleffects_internal",
             "afex_aov",
-            "amest", #package: Amelia
+            "amest", # package: Amelia
             "bart", # package: dbarts
             "betareg",
             "bglmerMod",
@@ -122,6 +128,7 @@ sanity_model_supported_class <- function(model) {
             "stanreg",
             "survreg",
             "svyolr",
+            "systemfit",
             c("tobit", "survreg"),
             "tobit1",
             "truncreg",
@@ -153,14 +160,12 @@ sanity_model_supported_class <- function(model) {
 }
 
 
-sanitize_model <- function(
-    model,
-    newdata = NULL,
-    vcov = NULL,
-    by = FALSE,
-    calling_function = NULL,
-    ...
-) {
+# Function that operates on mfx objects
+sanitize_model <- function(model, call, newdata = NULL, vcov = NULL, by = FALSE, ...) {
+    # Extract calling_function from mfx@call
+    calling_function <- extract_calling_function(call)
+
+    # Sanitize the model
     model <- sanitize_model_specific(
         model,
         vcov = vcov,
@@ -170,5 +175,7 @@ sanitize_model <- function(
         ...
     )
     sanity_model_supported_class(model)
+
+    # Assign the sanitized model to the slot
     return(model)
 }
